package org.jboss.qe.cxf.rs.providers;

import javax.ws.rs.Consumes;
import javax.ws.rs.ProcessingException;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyReader;
import javax.ws.rs.ext.Provider;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import java.io.IOException;
import java.io.InputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;

import lombok.extern.slf4j.Slf4j;

@Provider
@Slf4j
@Consumes(MediaType.APPLICATION_XML)
public class XmlMessageBodyReader implements MessageBodyReader<Message> {

	@Override
	public boolean isReadable(Class<?> aClass, Type type, Annotation[] annotations, MediaType mediaType) {
		return type == Message.class;
	}

	@Override
	public Message readFrom(Class<Message> aClass,
			Type type,
			Annotation[] annotations,
			MediaType mediaType,
			MultivaluedMap<String, String> multivaluedMap,
			InputStream inputStream)
			throws IOException, WebApplicationException {
		log.info("\nInvoke XmlMessageBodyReader...");
		try {
			final JAXBContext jaxbContext = JAXBContext.newInstance(Message.class);
			final Message message = (Message) jaxbContext.createUnmarshaller()
					.unmarshal(inputStream);
			return message;
		} catch (JAXBException jaxbException) {
			throw new ProcessingException("Error deserializing a Message.",
					jaxbException);
		}
	}
}
