package org.jboss.qe.cxf.rs.providers;

import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;

import java.io.IOException;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;

import lombok.extern.slf4j.Slf4j;

@Provider
@Slf4j
@Produces(MediaType.APPLICATION_XML)
public class XmlMessageBodyWriter implements MessageBodyWriter<Message> {

	@Override
	public boolean isWriteable(Class<?> aClass, Type type, Annotation[] annotations, MediaType mediaType) {
		return type == Message.class;
	}

	@Override
	public long getSize(Message message, Class<?> aClass, Type type, Annotation[] annotations, MediaType mediaType) {
		return 0;
	}

	@Override
	public void writeTo(Message message,
			Class<?> aClass,
			Type type,
			Annotation[] annotations,
			MediaType mediaType,
			MultivaluedMap<String, Object> multivaluedMap,
			OutputStream outputStream) throws IOException, WebApplicationException {
		log.info("\nInvoke XmlMessageBodyWriter...");
		try {
			final JAXBContext jaxbContext = JAXBContext.newInstance(Message.class);
			jaxbContext.createMarshaller().marshal(message, outputStream);
		} catch (JAXBException e) {
			e.printStackTrace();
		}
	}
}
